
class Knoten:
    """
    Beschreibung der Klasse Knoten
    """
    def __init__(self, dNeu, kNeu=None):
        """
        Konstruktor des Knoten
        :param dNeu: Datenelement
        :param kNeu: neuer Nachfolger (optional)
        """
        self.daten = dNeu
        self.nachfolger = kNeu

    def HintenEinfuegen(self, dNeu):
        """
        Fügt ein Datenelement am Ende der Liste ein
        :param dNeu: neues Datenelement
        """
        if self.nachfolger:
            self.nachfolger.HintenEinfuegen(dNeu)
        else:
            self.nachfolger = Knoten(dNeu)

    def RestlaengeGeben(self):
        """
        Bestimmt die Länge der restlichen Liste ab dem aktuellen Knoten
        :return: Länge der restlichen Liste
        """
        if not self.nachfolger:
            return 1
        return self.nachfolger.RestlaengeGeben() + 1

    def Suchen(self, gesuchterWert):
        """
        Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
        :param gesuchterWert: Zeichenkette, die als Schlüssel gefunden werden soll.
        :return: das gesuchte Datenelement
        """
        if self.daten.SchluesselIstGleich(gesuchterWert):
            return self.daten
        if self.nachfolger:
            return self.nachfolger.Suchen(gesuchterWert)
        return None

    def InformationAusgeben(self):
        """
        Gibt Information über das Datenelement aus.
        """
        self.daten.InformationAusgeben()
        if self.nachfolger:
            self.nachfolger.InformationAusgeben()

    def EinfuegenNach(self, dNeu, dVergleich):
        """
        Fügt ein Datenelement nach einem bestimmten anderen Datenelement ein
        :param dNeu: neues Datenelement
        :param dVergleich: gesuchtes Datenelement, vor dem eingefügt werden soll
        """
        if self.daten != dVergleich:
            if self.nachfolger:
                self.nachfolger.EinfuegenNach(dNeu, dVergleich)
            else:
                self.HintenEinfuegen(dNeu)
        else:
            kNeu = Knoten(dNeu, self.nachfolger)
            self.nachfolger = kNeu

    def KnotenEntfernen(self, dvergleich):
        """
        Entfernt einen Knoten, der über sein Datenelement identifiziert wird, aus der Liste
        :param dvergleich: gesuchtes Datenelement
        :return: Weitergabe des künftigen Nachfolgers
        """
        if self.daten == dvergleich:
            return self.nachfolger
        if self.nachfolger:
            self.nachfolger = self.nachfolger.KnotenEntfernen(dvergleich)
            return self
        return None

    def SortiertEinfuegen(self, dNeu):
        """
        Fügt ein Datenelement sortiert in die Liste ein
        :param dNeu: neues Datenelement
        :return: Weitergabe des künftigen Nachfolgers
        """
        if self.daten.IstKleinerAls(dNeu):
            if self.nachfolger:
                self.nachfolger = self.nachfolger.SortiertEinfuegen(dNeu)
            else:
                self.HintenEinfuegen(dNeu)
            return self
        else:
            kNeu = Knoten(dNeu, self)
            return kNeu

    def DatenelementGeben(self):
        """
        Gibt das Datenelement zurück (zur Anzeige)
        :return: daten
        """
        return self.daten

    def NachfolgerGeben(self):
        """
        Gibt den Nachfolger zurück (zur Anzeige)
        :return: nachfolger
        """
        return self.nachfolger


class Liste:
    """
    Beschreibung der Liste
    """
    def __init__(self):
        """
        Konstruktor der Liste
        """
        self.anfang = None

    def HintenEinfuegen(self, dNeu):
        """
        Fügt ein neues Datenelement am Ende der Liste ein.
        :param dNeu: neues Datenelement
        """
        if self.anfang:
            self.anfang.HintenEinfuegen(dNeu)
        else:
            self.anfang = Knoten(dNeu)

    def LaengeGeben(self):
        """
        Bestimmt die Länge der Liste
        :return: Listenlänge
        """
        if not self.anfang:
            return 0
        return self.anfang.RestlaengeGeben()

    def VorneEinfuegen(self, dNeu):
        """
        Fügt ein neues Datenelement am Anfang der Liste ein.
        :param dNeu: neues Datenelement
        """
        kNeu = Knoten(dNeu, self.anfang)
        self.anfang = kNeu

    def Suchen(self, gesuchterWert):
        """
        Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
        :param gesuchterWert: Zeichenkette, die als Schlüssel gefunden werden soll.
        :return: das gesuchte Datenelement
        """
        if self.anfang:
            return self.anfang.Suchen(gesuchterWert)
        return None

    def InformationAusgeben(self):
        """
        Gibt Information über die Liste aus.
        """
        if self.anfang:
            self.anfang.InformationAusgeben()

    def EinfuegenNach(self, dNeu, dVergleich):
        """
        Fügt ein neues Datenelement nach einem bestimmten Datenelement ein.
        :param dNeu: neues Datenelement
        :param dVergleich: Datenelement, nach dem eingefügt werden soll
        """
        if self.anfang:
            self.anfang.EinfuegenNach(dNeu, dVergleich)
        else:
            self.VorneEinfuegen(dNeu)

    def KnotenEntfernen(self, dVergleich):
        """
        Entfernt einen Knoten mit einem bestimmten Datenelement aus der Liste
        :param dVergleich: gesuchtes Datenelement
        """
        if self.anfang:
            self.anfang = self.anfang.KnotenEntfernen(dVergleich)

    def SortiertEinfuegen(self, dNeu):
        """
        Fügt ein neues Datenelement sortiert in die Liste ein.
        :param dNeu: neues Datenelement
        """
        if self.anfang:
            self.anfang = self.anfang.SortiertEinfuegen(dNeu)
        else:
            self.VorneEinfuegen(dNeu)

    def AnfangGeben(self):
        """
        Gibt den Anfang der Liste zurück (zur Anzeige)
        :return: anfang
        """
        return self.anfang
    
    def AnfangEntfernen(self):
        """
        Entfernt den Anfang der Liste und gibt das Datenelement zurück
        :return: erstes Datenelement
        """
        if not self.anfang is None:
            daten = self.AnfangGeben().DatenelementGeben()
            anfang = self.KnotenEntfernen(daten)
            return daten

